#include <errno.h>
#include <math.h>
#include <poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "xwiimote.h"

static struct xwii_iface *iface;
static int val2deg = 1.1945;
int offset_x = 0, offset_y = 0;

void error(const char *msg)
{
    perror(msg);
    exit(0);
}

int connection(char *argv[])
{
    int sockfd, portno;
    struct sockaddr_in serv_addr;
    struct hostent *server;

    portno = atoi(argv[2]);
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERROR opening socket");
    server = gethostbyname(argv[1]);
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr,
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);
    serv_addr.sin_port = htons(portno);
    if (connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0)
    {
        error("ERROR connecting");
        return (-1);
    }
    else
        return sockfd;
}

static char *get_dev(int num)
{
    struct xwii_monitor *mon;
    char *ent;
    int i = 0;

    mon = xwii_monitor_new(false, false);
    if (!mon) {
        printf("Cannot create monitor\n");
        return NULL;
    }

    while ((ent = xwii_monitor_poll(mon))) {
        if (++i == num)
            break;
        free(ent);
    }

    xwii_monitor_unref(mon);

    if (!ent)
        printf("Cannot find device with number #%d\n", num);

    return ent;
}

static void wii_event(const struct xwii_event *event, int sockfd)
{
    int n, reply[1];
    int arg[2] = {0};

    unsigned int code = event->v.key.code;
    bool pressed = event->v.key.state;

    arg[0] = event->v.abs[0].x*val2deg;
    arg[1] = event->v.abs[0].y*val2deg;

//    if(code == XWII_KEY_A)
//    {
//        if(pressed)
//        {
//            offset_x = event->v.abs[0].x;
//            offset_y = event->v.abs[0].y;
//            arg[3] = 1;
//        }
//        else
//            arg[3] = 0;
//    }

        //arg[0] = (event->v.abs[0].x - *offset_x)*val2deg;
        //arg[1] = (event->v.abs[0].y - *offset_y)*val2deg;
        //arg[2] = event->v.abs[0].z;

    //printf("Reset:%d Offset X:%d Offset Y:%d\n", arg[3], offset_x, offset_y);
    //printf("X:%d Y:%d Z:%d Reset:%d Offset X:%d Offset Y:%d\n", arg[0], arg[1], arg[2], arg[3], *offset_x, *offset_y);

    n = write(sockfd, arg, sizeof(arg));
    if (n < 0)
         error("ERROR writing to socket");
    bzero(reply,sizeof(reply));

    n = read(sockfd,reply,sizeof(reply));
    if (n < 0)
         error("ERROR reading from socket");
}

static int run_iface(struct xwii_iface *iface, int sockfd)
{
    struct xwii_event event;
    int ret = 0, fds_num;
    struct pollfd fds[2];

    memset(fds, 0, sizeof(fds));
    fds[0].fd = 0;
    fds[0].events = POLLIN;
    fds[1].fd = xwii_iface_get_fd(iface);
    fds[1].events = POLLIN;
    fds_num = 2;

    ret = xwii_iface_watch(iface, true);
    if (ret)
        printf("Error: Cannot initialize hotplug watch descriptor\n");

    while (1) {
        ret = poll(fds, fds_num, -1);
        if (ret < 0) {
            if (errno != EINTR) {
                ret = -errno;
                printf("Error: Cannot poll fds: %d\n", ret);
                break;
            }
        }

        ret = xwii_iface_dispatch(iface, &event, sizeof(event));
        if (ret) {
            if (ret != -EAGAIN) {
                printf("Error: Read failed with err:%d\n",
                        ret);
                break;
            }
        }
        wii_event(&event, sockfd);

    }
    return ret;
}

int main(int argc, char **argv)
{
    int ret = 0;
    char *path = NULL;
    int sockfd;

    if (argc < 4) {
       fprintf(stderr,"usage %s hostname port remote#\n", argv[0]);
       exit(0);
    }
    else
    {
        if (argv[3][0] != '/')
            path = get_dev(atoi(argv[3]));

        ret = xwii_iface_new(&iface, path ? path : argv[1]);
        free(path);

        if (ret)
            printf("Cannot create xwii_iface '%s' err:%d\n", argv[1], ret);
        else
        {
            ret = xwii_iface_open(iface, xwii_iface_available(iface) | XWII_IFACE_WRITABLE);
            if (ret)
                printf("Error: Cannot open interface: %d",ret);

              ret = xwii_iface_open(iface, XWII_IFACE_ACCEL);
              if (ret)
                  printf("Error: Cannot enable accelerometer: %d\n",ret);
              else
                  printf("Info: Enable accelerometer\n");

              ret = xwii_iface_open(iface, XWII_IFACE_CORE |
                               XWII_IFACE_WRITABLE);
              if (ret)
                  printf("Error: Cannot enable key events: %d\n", ret);
              else
                  printf("Info: Enable key events\n");

            sockfd = connection(argv);
            if(sockfd != -1)
                ret = run_iface(iface, sockfd);
            xwii_iface_unref(iface);
            if (ret)
                printf("Program failed; press any key to exit");
        }
    }

    return abs(ret);
}
